//
//
// DenyAllOption
//
//

// Ported to Windows95/NT by Jörgen Karlsson - d93-jka@nada.kh.se
// Date: June 1996

#include <Windows.h>
#include "DenyAllOption.h"

/*short
DenyAllOption::OptionFor (unsigned char)
{
	return 1;   // Accepts negotiation for ALL options!!!
}
*/
void DenyAllOption::NegotiateOption (unsigned char command)
{
	unsigned char buff[3];
	
	buff[0] = IAC;
	buff[1] = (command == DO) ? WONT : DONT;  // Deny all options
	stream->UngetChar ();            // put back option
	if (!stream->GetChar (buff+2))   // get option again
		return;
	
	SendReply (buff, 3);
	stream->MarkAsRead ();
}

void DenyAllOption::ExecOption(unsigned int Command)
{

}
	