#define		NUL				0x00
#define		BEL				0x07	// Bell
#define		BS				0x08	// Back Space
#define		HT				0x09	// Horizontal Tab
#define End 0x04
#define Enter 0x03
#define	Linefeed			 0x0A
#define Return 0x0D
#define ForwardDelete 0x7F
#define		ESC				0x1B
#define		SPACE			' '		// Space
#define Home				VK_HOME
#define UpArrow				VK_UP
#define DownArrow			VK_DOWN
#define LeftArrow			VK_LEFT
#define RightArrow			VK_RIGHT
#define PageUp 0x0B
#define PageDown 0x0C
