// TelnetApp.h
// Author: Jörgen Karlsson - d93-jka@nada.kth.se
#ifndef TELNETAPP_H

#ifndef TELNETSESSION_H
#include "TelnetSession.h"
#endif

extern HANDLE theEvent[1];
extern int EventCount;


class TelnetApp
{
public:
	static void Message(char *text);
	void CloseAllSessions(void);
	static void RegisterRecEvent(HANDLE RecEvent);
	char * GetHostName(char* defaultHost);
	void SessionClosed(TelnetSession *thisTelnetSession);
	static void Error(int ErrorCode, char *ErrorString);
	HINSTANCE AppInstance;
	TelnetApp(HINSTANCE hInst);
	~TelnetApp(void);
};

extern TelnetApp *theApp;
extern TelnetSession *theSession;

#endif // TELNETAPP_H
