// Telnet.h
// Author: Jörgen Karlsson - d93-jka@nada.kth.se
#ifndef TELNET_H
#define TELNET_H

#include "TelnetCodes.h"

typedef int					BOOL;
#define TRUE				1
#define FALSE				0


#endif // TELNET_H
