//
//
// DenyAllOption
//
// Denies all incoming options, use with care!!
//

#ifndef DENYALLOPTION__H
#define DENYALLOPTION__H

#include "CharStream.h"
#include "TelnetCodes.h"
#include "Option.h"

class DenyAllOption : public Option {
public:
	DenyAllOption (CharStream* stream, TelnetEngine* engine)
	: Option (stream, engine, 0) // 0 has no meaning here...
	{}
	void NegotiateOption (unsigned char);
	// short OptionFor (unsigned char);
	void ExecOption(unsigned int Command);
};

#endif