//
//
//  TelnetCodes.h
//
//

#ifndef TELNETCODES__H
#define TELNETCODES__H

// Telnet option definitions
#define 	BIN			0
#define 	ECHO		1
#define 	SGA			3

// Telnet commands
#define		SE		240
#define 	NOP		241
#define		DMARK 	242
#define     BRK		243
#define		IP		244
#define		AO		245
#define		AYT		246
#define		EC		247
#define		EL		248
#define		GA		249
#define     SB      250
#define 	WILL	251
#define 	WONT	252
#define 	DO 		253
#define 	DONT	254
#define 	IAC		255

#endif
