// TelnetSession.h
// Author: Jörgen Karlsson - d93-jka@nada.kth.se
#ifndef	TELNETSESSION_H
#define TELNETSESSION_H

#ifndef TELNETENGINE_H
class TelnetEngine;
#endif

#ifndef EMULATORENGINE_H
class EmulatorEngine;
#endif

//#ifndef CRYPTOENGINE_H
//#include "CryptoEngine.h"
//#endif

#include "CharStream.h"

class TelnetSession
{
public:
	void ConnectionBroken(void);
	void InvokeCommand(unsigned short command, void* Data = NULL);
	void Close(void);
	void RecCallback(CharStream *RecStream);
	BOOL Connect(char *hostname);
	TelnetSession(void);
	EmulatorEngine *mTerminalEngine;
	TelnetEngine *mTelnetEngine;
};


#endif // TELNETSESSION_H
