// krbmanagerView.cpp : implementation of the CKrbmanagerView class
//

#include "stdafx.h"
#include "krbmanager.h"

#include "krbmanagerDoc.h"
#include "krbmanagerView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CKrbmanagerView

IMPLEMENT_DYNCREATE(CKrbmanagerView, CView)

BEGIN_MESSAGE_MAP(CKrbmanagerView, CView)
	//{{AFX_MSG_MAP(CKrbmanagerView)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKrbmanagerView construction/destruction

CKrbmanagerView::CKrbmanagerView()
{
	// TODO: add construction code here

}

CKrbmanagerView::~CKrbmanagerView()
{
}

BOOL CKrbmanagerView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CKrbmanagerView drawing

void CKrbmanagerView::OnDraw(CDC* pDC)
{
	CKrbmanagerDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CKrbmanagerView diagnostics

#ifdef _DEBUG
void CKrbmanagerView::AssertValid() const
{
	CView::AssertValid();
}

void CKrbmanagerView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CKrbmanagerDoc* CKrbmanagerView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CKrbmanagerDoc)));
	return (CKrbmanagerDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CKrbmanagerView message handlers
