#ifdef HAVE_CONFIG_H
#include <config.h>
RCSID("$Id: ktypes.c,v 1.2 1997/05/18 18:47:44 assar Exp $");
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_BITYPES_H
#include <sys/bitypes.h>
#endif
#ifdef HAVE_BIND_BITYPES_H
#include <bind/bitypes.h>
#endif
#ifdef HAVE_NETINET_IN6_MACHTYPES_H
#include <netinet/in6_machtypes.h>
#endif

int
main(void)
{
    printf ("/*\n"
	    " * This file was automatically generated by $Id: ktypes.c,v 1.2 1997/05/18 18:47:44 assar Exp $.\n"
	    " * Please do not edit\n"
	    " */\n\n");

    printf ("#ifndef __KTYPES_H__\n"
	    "#define __KTYPES_H__\n\n");

#ifdef HAVE_SYS_TYPES_H
    printf("#include <sys/types.h>\n");
#endif
#ifdef HAVE_SYS_BITYPES_H
    printf("#include <sys/bitypes.h>\n");
#endif
#ifdef HAVE_BIND_BITYPES_H
    printf("#include <bind/bitypes.h>\n");
#endif
#ifdef HAVE_NETINET_IN6_MACHTYPES_H
    printf("#include <netinet/in6_machtypes.h>\n");
#endif
    
#ifndef HAVE_INT8_T
    printf("typedef signed char int8_t;\n");
#endif
#ifndef HAVE_U_INT8_T
    printf("typedef unsigned char u_int8_t;\n");
#endif
#ifndef HAVE_INT16_T
    printf("typedef short int16_t;\n");
#endif
#ifndef HAVE_U_INT16_T
    printf("typedef unsigned short u_int16_t;\n");
#endif
#ifndef HAVE_INT32_T
    printf("typedef int int32_t;\n");
#endif
#ifndef HAVE_U_INT32_T
    printf("typedef unsigned int u_int32_t;\n");
#endif

    printf("\n#endif /* __KTYPES_H__ */\n");
    return 0;
}
