//Microsoft Developer Studio generated resource script.
//
#include "resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#include "afxres.h"

/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// Swedish resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_SVE)
#ifdef _WIN32
LANGUAGE LANG_SWEDISH, SUBLANG_DEFAULT
#pragma code_page(1252)
#endif //_WIN32

/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_DIALOG1 DIALOG DISCARDABLE  0, 0, 186, 58
STYLE DS_ABSALIGN | DS_MODALFRAME | DS_CENTER | WS_POPUP | WS_CAPTION | 
    WS_SYSMENU
CAPTION "Enter a hostname"
FONT 8, "MS Sans Serif"
BEGIN
    EDITTEXT        IDC_EDIT1,25,15,121,14,ES_AUTOHSCROLL
    DEFPUSHBUTTON   "OK",IDOK,25,37,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,96,37,50,14
END


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO DISCARDABLE 
BEGIN
    IDD_DIALOG1, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 179
        TOPMARGIN, 7
        BOTTOMMARGIN, 51
    END
END
#endif    // APSTUDIO_INVOKED


#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE DISCARDABLE 
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE DISCARDABLE 
BEGIN
    "#include ""afxres.h""\r\n"
    "\0"
END

3 TEXTINCLUDE DISCARDABLE 
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Menu
//

IDR_MENU_TELNET MENU DISCARDABLE 
BEGIN
    POPUP "Connection"
    BEGIN
        MENUITEM "Close",                       ID_CONNECTION_CLOSE
    END
    POPUP "Telnet"
    BEGIN
        MENUITEM "Send",                        ID_TELNET_SEND, GRAYED
        MENUITEM """Are You There""",           ID_TELNET_SEND_AYT
        MENUITEM """Break""",                   ID_TELNET_SEND_BRK
        MENUITEM """Interupt Process""",        ID_TELNET_SEND_IP
        MENUITEM """Abort Output""",            ID_TELNET_SEND_AO
        MENUITEM """Erase Character""",         ID_TELNET_SEND_EC
        MENUITEM """Erase Line""",              ID_TELNET_SEND_EL
    END
    POPUP "Encryption"
    BEGIN
        MENUITEM "On",                          ID_ENCRYPTION_ON
        MENUITEM "Off",                         ID_ENCRYPTION_OFF
    END
    POPUP "About"
    BEGIN
        MENUITEM "Show Version",                ID_SHOWVERSION
    END
END


#ifndef _MAC
/////////////////////////////////////////////////////////////////////////////
//
// Version
//

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 1,0,0,1
 PRODUCTVERSION 1,0,0,1
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x40004L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "041d04b0"
        BEGIN
            VALUE "CompanyName", "KTH\0"
            VALUE "FileDescription", "Telnet\0"
            VALUE "FileVersion", "1, 0, 0, 1\0"
            VALUE "InternalName", "Telnet\0"
            VALUE "LegalCopyright", "Copyright © 1996\0"
            VALUE "OriginalFilename", "Telnet.exe\0"
            VALUE "ProductName", "KTH Telnet\0"
            VALUE "ProductVersion", "1, 0, 0, 1\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x41d, 1200
    END
END

#endif    // !_MAC

#endif    // Swedish resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

